#include <LiquidCrystal.h>

// LCD digital connection pins (RS, E, D4, D5, D6, D7)
LiquidCrystal lcd(2, 3, 4, 5, 6, 7);

int analogPin = A0; // analog pin
float Vin = 5.0;
float Vout = 0.0;   // voltage drop on reference resistor
float R_1k = 1000.0;     // reference resistor
float R_unknown = 0.0;   // unknown resistor
int LCD_value = 0;   // analog to digital converter value / LCD value

void setup() {
  lcd.begin(16, 2); 
  lcd.setCursor(0, 0); // adjusting rows and columns on the LCD screen (row, column)
  lcd.print(" Ohm Meter ");
  delay(2000); // show Ohmmeter 2 sec
  lcd.clear(); 
}

void loop() {
  LCD_value = analogRead(analogPin); // analog pin read

  // open circuit control
  if (LCD_value == 0) {
    lcd.setCursor(0, 0);
    lcd.print("Open Circuit");
    lcd.setCursor(0, 1);
    lcd.print("Connection Error");
  } else {
    Vout = (LCD_value * Vin) / 1023.0; // convert analog value to voltage

    R_unknown = (R_1k * (Vin - Vout)) / Vout; // calculate unknown resistor

    // Check for resistance limits
    if (R_unknown < 180.0 || R_unknown > 33000.0) {
      lcd.setCursor(0, 0);
      lcd.print("Error: Out of");
      lcd.setCursor(0, 1);
      lcd.print("Range");
    } else {
      lcd.setCursor(0, 0); 
      lcd.print("Resistance: ");

      // kOhm - Ohm detector
      lcd.setCursor(0, 1); 
      if (R_unknown > 1000.0) {
        lcd.print(R_unknown / 1000.0, 2); // 2 decimal shown
        lcd.print(" kOhm"); 
      } else {
        lcd.print(R_unknown, 2);
        lcd.print(" Ohm");
      }
    }
  }
  delay(1000); // refresh sec
}
